/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package net.jazz.rtcext.workitem.extensions.ide.ui;

import net.jazz.rtcext.workitem.extensions.common.IBuildOnStateChangeDefinitions;

import com.ibm.team.foundation.common.util.IMemento;

/**
 * The model class is used to read and write XML from and to a project or team
 * area's process specification. It also provides get and set methods for the
 * fields for use by the {@link BuildOnStateChangeAspectEditor} aspect editor.
 * Note that the set methods return true if the values actually changed. They
 * return false if the value did not change. Also note that the id fields are
 * never null.
 */
public class BuildOnStateChangeModel {

	/*
	 * The work item type, state and build definition ids are stored in these
	 * fields. All access should be through the get an set methods. In
	 * particular, the read from and write to methods use the get and set
	 * methods to ensure consistency in how the values are normalized.
	 */
	private String fWorkItemTypeId = IBuildOnStateChangeDefinitions.EMPTY_ID;
	private String fWorkItemStateId = IBuildOnStateChangeDefinitions.EMPTY_ID;
	private String fBuildDefId = IBuildOnStateChangeDefinitions.EMPTY_ID;

	/**
	 * Get access method for the work item type id.
	 * 
	 * @return the workItemTypeId
	 */
	public String getWorkItemTypeId() {
		return fWorkItemTypeId;
	}

	/**
	 * Set access method for the work item type id. The id is normalized and
	 * true is returned if a changes is actually made.
	 * 
	 * @param workItemTypeId
	 *            the work item type id to set
	 * @return true if the value changed, false if it did not
	 */
	public boolean setWorkItemTypeId(String workItemTypeId) {
		boolean changed = false;
		String normalizedId = normalize(workItemTypeId);
		if (!fWorkItemTypeId.equals(normalizedId)) {
			fWorkItemTypeId = normalizedId;
			changed = true;
		}
		return changed;
	}

	/**
	 * Get access method for the work item state id.
	 * 
	 * @return the workItemStateId
	 */
	public String getWorkItemStateId() {
		return fWorkItemStateId;
	}

	/**
	 * Set access method for the work item state id. The id is normalized and
	 * true is returned if a changes is actually made.
	 * 
	 * @param workItemStateId
	 *            the work item state id to set
	 * @return true if the value changed, false if it did not
	 */
	public boolean setWorkItemStateId(String workItemStateId) {
		boolean changed = false;
		String normalizedId = normalize(workItemStateId);
		if (!fWorkItemStateId.equals(normalizedId)) {
			fWorkItemStateId = normalizedId;
			changed = true;
		}
		return changed;
	}

	/**
	 * Get access method for the build definition id.
	 * 
	 * @return the buildDefId
	 */
	public String getBuildDefId() {
		return fBuildDefId;
	}

	/**
	 * Set access method for the build definition id. The id is normalized and
	 * true is returned if a changes is actually made.
	 * 
	 * @param buildDefId
	 *            the build definition id to set
	 * @return true if the value changed, false if it did not
	 */
	public boolean setBuildDefId(String buildDefId) {
		boolean changed = false;
		String normalizedId = normalize(buildDefId);
		if (!fBuildDefId.equals(normalizedId)) {
			fBuildDefId = normalizedId;
			changed = true;
		}
		return changed;
	}

	/**
	 * Normalizes null ids to the empty id and trims others.
	 * 
	 * @param string
	 *            the id to normalize
	 * @return the normalized id
	 */
	protected String normalize(String string) {
		return string == null ? IBuildOnStateChangeDefinitions.EMPTY_ID
				: string.trim();
	}

	/**
	 * The process framework parses the XML and passes us the root memento for
	 * our part of the process XML. From it, this method finds the trigger and
	 * build children and then their sub-elements that contain the ids.
	 * 
	 * @param memento
	 *            the aspect's (aka parent) memento
	 */
	public void readFrom(IMemento memento) {
		/*
		 * Find and process the trigger elements. Store the values using the
		 * field set methods to have consistent normalization.
		 */
		IMemento triggerMemento = memento
				.getChild(IBuildOnStateChangeDefinitions.TAG_TRIGGER);
		if (triggerMemento != null) {
			IMemento typeIdMemento = triggerMemento
					.getChild(IBuildOnStateChangeDefinitions.TAG_CHANGED_WORKITEM_TYPE);
			if (typeIdMemento != null) {
				setWorkItemTypeId(typeIdMemento
						.getString(IBuildOnStateChangeDefinitions.ID));
			}
			IMemento stateIdMemento = triggerMemento
					.getChild(IBuildOnStateChangeDefinitions.TAG_TRIGGER_STATE_ID);
			if (stateIdMemento != null) {
				setWorkItemStateId(stateIdMemento
						.getString(IBuildOnStateChangeDefinitions.ID));
			}
		}

		/*
		 * Find and process the build elements. Store the value using the field
		 * set method to have consistent normalization.
		 */
		IMemento buildMemento = memento
				.getChild(IBuildOnStateChangeDefinitions.TAG_BUILD);
		if (buildMemento != null) {
			IMemento buildDefIdMemento = buildMemento
					.getChild(IBuildOnStateChangeDefinitions.TAG_BUILD_DEFINITION);
			if (buildDefIdMemento != null) {
				setBuildDefId(buildDefIdMemento
						.getString(IBuildOnStateChangeDefinitions.ID));
			}
		}
	}

	/**
	 * The process framework passes us the root memento for our part of the
	 * process XML. This method then creates child trigger and build elements
	 * along with their sub-elements with the ids. The process framework
	 * eventually writes the XML.
	 * 
	 * @param memento
	 *            the aspect's (aka parent) memento
	 * @return true if the save worked
	 */
	public boolean saveTo(IMemento memento) {
		/*
		 * Create the mementos for the trigger elements. Obtain the values using
		 * the field get methods to have consistent normalization.
		 */
		IMemento triggerMemento = memento
				.createChild(IBuildOnStateChangeDefinitions.TAG_TRIGGER);
		IMemento child = triggerMemento
				.createChild(IBuildOnStateChangeDefinitions.TAG_CHANGED_WORKITEM_TYPE);
		child.putString(IBuildOnStateChangeDefinitions.ID, getWorkItemTypeId());
		child = triggerMemento
				.createChild(IBuildOnStateChangeDefinitions.TAG_TRIGGER_STATE_ID);
		child
				.putString(IBuildOnStateChangeDefinitions.ID,
						getWorkItemStateId());

		/*
		 * Create the mementos for the build elements. Obtain the value using
		 * the field get method to have consistent normalization.
		 */
		IMemento buildMemento = memento
				.createChild(IBuildOnStateChangeDefinitions.TAG_BUILD);
		child = buildMemento
				.createChild(IBuildOnStateChangeDefinitions.TAG_BUILD_DEFINITION);
		child.putString(IBuildOnStateChangeDefinitions.ID, getBuildDefId());

		return true;
	}
}
